div.admonition, div.attention, div.caution, div.danger, div.error,
div.hint, div.important, div.note, div.tip, div.warning, div.sidebar {
/* stolen from Boostrap 3 (.panel .panel-default) */
  margin-bottom: 20px;
  background-color: #fff;
  border: 1px solid #ddd;
  border-radius: 4px;
  -webkit-box-shadow: 0 1px 1px rgba(0, 0, 0, .05);
          box-shadow: 0 1px 1px rgba(0, 0, 0, .05);
  padding: 0 15px 15px 15px;
}

div.attention, div.caution, div.danger, div.error, div.warning {
  /* stolen from Boostrap 3 (.panel .panel-danger) */
  border-color: #EBCCD1;
}

div.admonition p.admonition-title, div.hint p.admonition-title,
div.important p.admonition-title, div.note p.admonition-title,
div.tip p.admonition-title, div.sidebar p.sidebar-title,
div.attention p.admonition-title, div.caution p.admonition-title,
div.danger p.admonition-title, div.error p.admonition-title,
div.warning p.admonition-title {
/* stolen from Boostrap 3 (.panel .panel-default .panel-heading) */
  font-size: 16px;
  color: #333;
  background-color: #F5F5F5;
  padding: 10px 15px;
  margin-left: -15px;
  margin-right: -15px;
  border-bottom: 1px solid rgba(0, 0, 0, 0);
  border-top-left-radius: 3px;
  border-top-right-radius: 3px;
  color: #333;
  background-color: #F5F5F5;
  border-color: #DDD;
}

div.attention p.admonition-title, div.caution p.admonition-title,
div.danger p.admonition-title, div.error p.admonition-title,
div.warning p.admonition-title {
  /* stolen from Boostrap 3 (.panel .panel-danger) */
  color: #A94442;
  background-color: #F2DEDE;
  border-color: #EBCCD1;
}
